@@ -1,18 +1,21 @@
 import pytest
+
+import rnalysis.utils.io
 from rnalysis.utils import io
 from rnalysis.utils.io import *
 from rnalysis.utils.io import _format_ids_iter, _ensmbl_lookup_post_request
 
 
 class MockResponse(object):
-    def __init__(self, status_code: int = 200, url: str = 'http://httpbin.org/get', headers: dict = {'blaa': '1234'},
-                 text: str = '', json_output: dict = dict()):
+    def __init__(self, status_code: int = 200, url: str = 'http://httpbin.org/get', headers: dict = 'default',
+                 text: str = '', json_output: dict = dict(), content: str = ''):
         self.status_code = status_code
         self.url = url
-        self.headers = headers
+        self.headers = {'default': 'default'} if headers == 'default' else headers
         self.text = text
         self.ok = self.status_code == 200
         self._json = json_output
+        self.content = bytes(content, 'utf8')
 
     def raise_for_status(self):
         if not self.ok:
@@ -21,6 +24,12 @@ class MockResponse(object):
     def json(self):
         return self._json
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        return
+
 
 def test_load_csv_bad_input():
     invalid_input = 2349
@@ -154,15 +163,18 @@ def test_golr_annotation_iterator_parse_evidence_types(monkeypatch, test_input,
 def test_golr_annotation_iterator_get_n_annotations(monkeypatch):
     num_found_truth = 126311
 
-    def fake_request(self, params):
+    def fake_request(self, params, cached_filename):
         assert isinstance(self, GOlrAnnotationIterator)
         assert isinstance(params, dict)
+        assert cached_filename == 'test.json'
         with open(f'tests/test_files/golr_header.txt') as f:
             return f.readline()
 
     monkeypatch.setattr(GOlrAnnotationIterator, '_golr_request', fake_request)
+    monkeypatch.setattr(GOlrAnnotationIterator, '_generate_cached_filename', lambda self, start: 'test.json')
     golr = GOlrAnnotationIterator.__new__(GOlrAnnotationIterator)
     golr.default_params = {}
+
     assert golr._get_n_annotations() == num_found_truth
 
 
@@ -195,7 +207,17 @@ def test_golr_annotation_iterator_golr_request_connectivity(monkeypatch):
     assert isinstance(GOlrAnnotationIterator._golr_request(fake_params), str)
 
 
+def remove_cached_test_file(cached_filename: str):
+    try:
+        os.remove(io.get_todays_cache_dir().joinpath(cached_filename))
+    except FileNotFoundError:
+        pass
+
+
 def test_golr_annotation_iterator_golr_request(monkeypatch):
+    cached_filename = 'test.json'
+    remove_cached_test_file(cached_filename)
+
     correct_url = GOlrAnnotationIterator.URL
     correct_params = {'param': 'value', 'other_param': 'other_value'}
 
@@ -205,7 +227,17 @@ def test_golr_annotation_iterator_golr_request(monkeypatch):
         return MockResponse(text='the correct text')
 
     monkeypatch.setattr(requests, 'get', mock_get)
-    assert GOlrAnnotationIterator._golr_request(correct_params) == 'the correct text'
+    monkeypatch.setattr(GOlrAnnotationIterator, '_generate_cached_filename', lambda self, start: 'test.json')
+    assert GOlrAnnotationIterator._golr_request(correct_params, cached_filename) == 'the correct text'
+
+    def mock_get_uncached(url, params: dict):
+        raise AssertionError("This function should not be called if a cached file was found!")
+
+    monkeypatch.setattr(requests, 'get', mock_get_uncached)
+    try:
+        assert GOlrAnnotationIterator._golr_request(correct_params, cached_filename) == 'the correct text'
+    finally:
+        remove_cached_test_file(cached_filename)
 
     def mock_get_failed(url, params: dict):
         assert url == correct_url
@@ -213,8 +245,11 @@ def test_golr_annotation_iterator_golr_request(monkeypatch):
         return MockResponse(text='the correct text', status_code=404)
 
     monkeypatch.setattr(requests, 'get', mock_get_failed)
-    with pytest.raises(ConnectionError):
-        _ = GOlrAnnotationIterator._golr_request(correct_params)
+    try:
+        with pytest.raises(ConnectionError):
+            _ = GOlrAnnotationIterator._golr_request(correct_params)
+    finally:
+        remove_cached_test_file(cached_filename)
 
 
 def test_golr_annotation_iterator_parsing(monkeypatch):
@@ -234,13 +269,15 @@ def test_golr_annotation_iterator_parsing(monkeypatch):
                      {"source": "WB", "bioentity_internal_id": "WBGene00011481", "annotation_class": "GO:0005783"},
                      {"source": "WB", "bioentity_internal_id": "WBGene00011481", "annotation_class": "GO:0005789"}]
 
-    def fake_request(self, params):
+    def fake_request(self, params, cached_filename):
         assert isinstance(self, GOlrAnnotationIterator)
         assert params == truth_params
+        assert cached_filename == 'test.json'
         with open(f'tests/test_files/golr_response.txt') as f:
             return f.readline()
 
     monkeypatch.setattr(GOlrAnnotationIterator, '_golr_request', fake_request)
+    monkeypatch.setattr(GOlrAnnotationIterator, '_generate_cached_filename', lambda self, start: 'test.json')
 
     request_params = {
         "q": "*:*",
@@ -271,9 +308,9 @@ def test_map_taxon_id(monkeypatch):
     taxon_name = 'c elegans'
 
     def mock_requests_get(url, params):
-        assert url == 'https://www.uniprot.org/taxonomy/?'
-        assert params == {'format': 'tab', 'query': taxon_name, 'sort': 'score'}
-        return MockResponse(text='Taxon\tMnemonic\tScientific name\tCommon name\tSynonym\tOther Names\tReviewed\t'
+        assert url == 'https://rest.uniprot.org/taxonomy/search?'
+        assert params == {'format': 'tsv', 'query': taxon_name}
+        return MockResponse(text='Taxon Id\tMnemonic\tScientific name\tCommon name\tSynonym\tOther Names\tReviewed\t'
                                  'Rank\tLineage\tParent\tVirus hosts\n6239\tCAEEL\tCaenorhabditis elegans\t\t\t'
                                  'Caenorhabditis elegans (Maupas, 1900); Rhabditis elegans; Rhabditis elegans Maupas, '
                                  '1900; roundworm\treviewed\tSpecies\tEukaryota; Metazoa; Ecdysozoa; Nematoda; '
@@ -296,7 +333,7 @@ def test_map_taxon_id_no_results(monkeypatch):
 def test_map_taxon_id_multiple_results(monkeypatch):
     def mock_requests_get(url, params):
         return MockResponse(
-            text='Taxon\tScientific name\n9615\tCanis lupus familiaris\n2509620\t'
+            text='Taxon Id\tScientific name\n9615\tCanis lupus familiaris\n2509620\t'
                  'Wlobachia endosymbiont of Canis lupus familiaris\n990119\tCanis lupus x Canis lupus familiaris')
 
     monkeypatch.setattr(requests, 'get', mock_requests_get)
@@ -347,7 +384,7 @@ def test_infer_sources_from_gene_ids(monkeypatch, gene_id_info, truth):
 def test_infer_taxon_from_gene_ids(monkeypatch, gene_id_info, truth):
     monkeypatch.setattr(io, 'map_taxon_id', lambda x: x)
     monkeypatch.setattr(io, '_ensmbl_lookup_post_request', lambda x: gene_id_info)
-    assert infer_taxon_from_gene_ids([]) == truth
+    assert infer_taxon_from_gene_ids([])[0] == truth
 
 
 def test_infer_taxon_from_gene_ids_no_species(monkeypatch):
@@ -386,8 +423,9 @@ def test_map_gene_ids_connectivity():
         assert mapped_ids[geneid] == wb_to_entrez_truth[geneid]
 
 
-def test_map_gene_ids_to_same_set():
-    mapper = map_gene_ids(['it', 'doesnt', 'matter', 'what', 'is', 'in', 'here'], 'UniProtKB', 'UniProtKB')
+@pytest.mark.parametrize('id_type', ['UniProtKB', 'Entrez', 'WormBase'])
+def test_map_gene_ids_to_same_set(id_type):
+    mapper = map_gene_ids(['it', 'doesnt', 'matter', 'what', 'is', 'in', 'here'], id_type, id_type)
     assert mapper.mapping_dict is None
     for i in ['it', 'not', False, 42, 3.14]:
         assert i in mapper
@@ -418,23 +456,288 @@ def test_map_gene_ids_request(monkeypatch, ids, map_from, map_to, req_from, req_
 
 @pytest.mark.parametrize('ids,map_from,map_to,txt,rev_txt,truth',
                          [(['WBGene00000003', 'WBGene00000004'], 'WormBase', 'UniProtKB',
-                           'Entry\tAnnotation\tyourlist:M202105238471C63D39733769F8E060B506551E1220FD37S\nQ19151\t1 out of 5\tWBGene00000003\nA0A0K3AVL7\t1 out of 5\tWBGene00000004\nO17395\t2 out of 5\tWBGene00000004\n'
+                           'From\tEntry\tAnnotation\nWBGene00000003\tQ19151\t110\nWBGene00000004\tA0A0K3AVL7\t57\nWBGene00000004\tO17395\t137.2\n'
                            , '', {'WBGene00000003': 'Q19151', 'WBGene00000004': 'O17395'}),
                           (
                               ['id1', 'id2'], 'UniProtKB', 'WormBase',
                               'From\tTo\nid1\tWBID1\nid2\tWBID2.2\nid2\tWBID2.1\n',
-                              'Entry\tAnnotation\tyourlist:M202105238471C63D39733769F8E060B506551E1220FD37S\nWBID1\t1 out of 5\tid1\nWBID2.1\t1 out of 5\tid2\nWBID2.2\t2 out of 5\tid2\n'
+                              'From\tEntry\tAnnotation\nWBID1\tid1\t112.5\nWBID2.1\tid2\t112.5\nWBID2.2\tid2\t235\n'
                               , {'id1': 'WBID1', 'id2': 'WBID2.2'})
                           ])
 def test_map_gene_ids_with_duplicates(monkeypatch, ids, map_from, map_to, txt, rev_txt, truth):
-    def mock_get(url, params):
-        if params['to'] == 'ACC':
-            return_txt = txt if map_to == 'UniProtKB' else rev_txt
-        else:
-            return_txt = txt if map_from == 'UniProtKB' else rev_txt
-        return MockResponse(text=return_txt)
+    def mock_abbrev_dict():
+        d = {'WormBase': 'WormBase',
+             'UniProtKB_to': 'UniProtKB',
+             'UniProtKB_from': 'UniProtKB_AC-ID',
+             'UniProtKB': 'UniProtKB'}
+        return d, d
 
-    monkeypatch.setattr(requests, 'get', mock_get)
+    def mock_get_mapping_results(api_url: str, from_db: str, to_db: str, ids: List[str], polling_interval: float,
+                                 session, verbose):
+        mock_abbrev_dict_to, mock_abbrev_dict_from = mock_abbrev_dict()
+        if to_db == mock_abbrev_dict_to['UniProtKB_to']:
+            return_txt = txt if map_to == 'UniProtKB' else rev_txt
+        elif from_db == mock_abbrev_dict_from['UniProtKB_from']:
+            return_txt = txt if map_from == 'UniProtKB' else rev_txt
+        else:
+            raise ValueError(to_db, from_db)
+        return return_txt.split('\n')
+
+    monkeypatch.setattr(io, '_get_id_abbreviation_dicts', mock_abbrev_dict)
+    monkeypatch.setattr(io, 'get_mapping_results', mock_get_mapping_results)
     res = map_gene_ids(ids, map_from, map_to)
     for gene_id in truth:
         assert res[gene_id] == truth[gene_id]
+
+
+def test_get_todays_cache_dir():
+    today = date.today()
+    today_str = str(today.year) + '_' + str(today.month).zfill(2) + '_' + str(today.day).zfill(2)
+    cache_dir_truth = os.path.join(appdirs.user_cache_dir('RNAlysis'), today_str)
+    assert cache_dir_truth == str(io.get_todays_cache_dir())
+
+
+def test_load_cached_file():
+    cached_filename = 'test.txt'
+    remove_cached_test_file(cached_filename)
+
+    cache_content_truth = "testing\n123"
+    cache_dir = io.get_todays_cache_dir()
+    path = os.path.join(cache_dir, cached_filename)
+
+    assert load_cached_file(cached_filename) is None
+
+    with open(path, 'x') as f:
+        f.write(cache_content_truth)
+
+    try:
+        assert load_cached_file(cached_filename) == cache_content_truth
+    finally:
+        remove_cached_test_file(cached_filename)
+
+
+def test_cache_file():
+    cached_filename = 'test.txt'
+    remove_cached_test_file(cached_filename)
+
+    cache_content_truth = "testing\n123"
+    cache_dir = io.get_todays_cache_dir()
+    path = os.path.join(cache_dir, cached_filename)
+
+    cache_file(cache_content_truth, cached_filename)
+    try:
+        with open(path, 'r') as f:
+            assert f.read() == cache_content_truth
+    finally:
+        remove_cached_test_file(cached_filename)
+
+
+@pytest.mark.parametrize("gene_set,expected_split", [
+    ({1, 2, 3}, ['1', '2', '3']),
+    ({'geneA', 'geneB', 'geneC', 'geneD'}, ["geneA", "geneB", "geneC", "geneD"])
+])
+def test_save_gene_set(gene_set, expected_split):
+    pth = 'tests/test_files/tmp_saved_gene_set.txt'
+    try:
+        save_gene_set(gene_set, pth)
+        with open(pth) as f:
+            split = f.read().split('\n')
+        assert sorted(split) == sorted(expected_split)
+    finally:
+        try:
+            os.unlink(pth)
+        except FileNotFoundError:
+            pass
+
+
+def test_kegg_annotation_iterator_api():
+    _ = KEGGAnnotationIterator(6239)
+
+
+@pytest.mark.parametrize('arguments,url_truth', [('argument', 'https://rest.kegg.jp/operation/argument'),
+                                                 (['arg1', 'arg2', 'arg3'],
+                                                  'https://rest.kegg.jp/operation/arg1/arg2/arg3'),
+                                                 (['argument'], 'https://rest.kegg.jp/operation/argument')], )
+def test_kegg_annotation_iterator_kegg_request(monkeypatch, arguments, url_truth):
+    truth = '{"sample": "json", "lorem": "ipsum"}'
+    cached = []
+
+    def mock_get_cached_file(filename):
+        return None
+
+    def mock_cache_file(content, filename):
+        assert filename == 'cached_filename.csv'
+        assert content == truth
+        cached.append(True)
+
+    def mock_get(url):
+        assert url == url_truth
+        return MockResponse(text=truth)
+
+    monkeypatch.setattr(io, 'load_cached_file', mock_get_cached_file)
+    monkeypatch.setattr(requests, 'get', mock_get)
+    monkeypatch.setattr(io, 'cache_file', mock_cache_file)
+    assert KEGGAnnotationIterator._kegg_request('operation', arguments, 'cached_filename.csv') == (
+        truth, False)
+    assert cached == [True]
+
+
+def test_kegg_annotation_iterator_kegg_request_cached(monkeypatch):
+    truth = {"sample": "json", "lorem": "ipsum"}
+
+    def mock_get_cached_file(filename):
+        return truth
+
+    monkeypatch.setattr(io, 'load_cached_file', mock_get_cached_file)
+    assert KEGGAnnotationIterator._kegg_request('operation', 'argument', 'cached_filename.csv') == (truth, True)
+
+
+PATHWAY_NAMES_TRUTH = {'path:cel00010': 'Glycolysis / Gluconeogenesis - Caenorhabditis elegans (nematode)',
+                       'path:cel00020': 'Citrate cycle (TCA cycle) - Caenorhabditis elegans (nematode)',
+                       'path:cel00030': 'Pentose phosphate pathway - Caenorhabditis elegans (nematode)',
+                       'path:cel00040': 'Pentose and glucuronate interconversions - Caenorhabditis elegans (nematode)',
+                       'path:cel00051': 'Fructose and mannose metabolism - Caenorhabditis elegans (nematode)',
+                       'path:cel00052': 'Galactose metabolism - Caenorhabditis elegans (nematode)', }
+
+
+def test_kegg_annotation_iterator_get_pathways(monkeypatch):
+    truth = (PATHWAY_NAMES_TRUTH, 6)
+    organism_code = 'cel'
+
+    def mock_kegg_request(self, operation, arguments):
+        assert operation == 'list'
+        assert arguments == ['pathway', organism_code]
+        with open('tests/test_files/kegg_pathways.txt') as f:
+            return f.read(), False
+
+    monkeypatch.setattr(KEGGAnnotationIterator, '_kegg_request', mock_kegg_request)
+
+    kegg = KEGGAnnotationIterator.__new__(KEGGAnnotationIterator)
+    kegg.organism_code = organism_code
+    assert kegg.get_pathways() == truth
+
+
+def test_kegg_annotation_iterator_get_pathway_annotations(monkeypatch):
+    truth = {'path:cel00010': ['Glycolysis / Gluconeogenesis - Caenorhabditis elegans (nematode)',
+                               {'cel:CELE_F14B4.2', 'cel:CELE_Y87G2A.8', 'cel:CELE_C50F4.2', 'cel:CELE_Y71H10A.1'}],
+             'path:cel00020': ['Citrate cycle (TCA cycle) - Caenorhabditis elegans (nematode)',
+                               {'cel:CELE_T20G5.2', 'cel:CELE_B0365.1', 'cel:CELE_D1005.1'}],
+             'path:cel00030': ['Pentose phosphate pathway - Caenorhabditis elegans (nematode)',
+                               {'cel:CELE_Y87G2A.8', 'cel:CELE_B0035.5'}],
+             'path:cel00040': ['Pentose and glucuronate interconversions - Caenorhabditis elegans (nematode)',
+                               {'cel:CELE_Y105E8B.9', 'cel:CELE_B0310.5', 'cel:CELE_T04H1.7', 'cel:CELE_T04H1.8'}],
+             'path:cel00051': ['Fructose and mannose metabolism - Caenorhabditis elegans (nematode)',
+                               {'cel:CELE_C05C8.7', 'cel:CELE_ZK632.4'}],
+             'path:cel00052': ['Galactose metabolism - Caenorhabditis elegans (nematode)', {'cel:CELE_C01B4.6'}]}
+    args_truth = ['path:cel00010+path:cel00020+path:cel00030', 'path:cel00040+path:cel00051+path:cel00052']
+
+    def mock_kegg_request(self, operation, arguments, fname):
+        assert operation == 'get'
+        assert arguments == args_truth[0] or arguments == args_truth[1]
+        if arguments == args_truth[0]:
+            pth = 'tests/test_files/kegg_annotation_1of2.txt'
+        else:
+            pth = 'tests/test_files/kegg_annotation_2of2.txt'
+        with open(pth) as f:
+            return f.read(), False
+
+    monkeypatch.setattr(KEGGAnnotationIterator, '_kegg_request', mock_kegg_request)
+    monkeypatch.setattr(KEGGAnnotationIterator, 'REQ_MAX_ENTRIES', 3)
+    kegg = KEGGAnnotationIterator.__new__(KEGGAnnotationIterator)
+    kegg.pathway_annotations = None
+    kegg.taxon_id = 6239
+    kegg.organism_code = 'cel'
+    kegg.pathway_names = PATHWAY_NAMES_TRUTH
+    assert {key: [name, ann] for key, name, ann in kegg.get_pathway_annotations()} == truth
+
+
+def test_kegg_annotation_iterator_get_pathway_annotations_cached():
+    annotation = {'a': 1, 'b': 2, 'c': 3}
+    names = {'a': 'namea', 'b': 'nameb', 'c': 'namec'}
+    truth = {'a': ['namea', 1], 'b': ['nameb', 2], 'c': ['namec', 3]}
+    kegg = KEGGAnnotationIterator.__new__(KEGGAnnotationIterator)
+    kegg.pathway_annotations = annotation
+    kegg.pathway_names = names
+    assert {key: [name, ann] for key, name, ann in kegg.get_pathway_annotations()} == truth
+
+
+@pytest.mark.parametrize('taxon_id,truth', [
+    (6239, 'cel'),
+    (270351, 'maqu'),
+    (9606, 'hsa'),
+    (6238, 'cbr')
+])
+def test_kegg_annotation_iterator_get_kegg_organism_code(monkeypatch, taxon_id, truth):
+    def mock_get_tree():
+        with open('tests/test_files/kegg_taxon_tree_truth.json') as f:
+            return json.loads(f.read())
+
+    monkeypatch.setattr(KEGGAnnotationIterator, '_get_taxon_tree', mock_get_tree)
+
+    assert KEGGAnnotationIterator.get_kegg_organism_code(taxon_id) == truth
+
+
+def test_kegg_annotation_iterator_get_taxon_tree(monkeypatch):
+    truth = {"sample": "json", "lorem": "ipsum"}
+    truth_text = '{"sample": "json", "lorem": "ipsum"}'
+    cached = []
+
+    def mock_get_cached_file(filename):
+        return None
+
+    def mock_cache_file(content, filename):
+        assert filename == 'kegg_taxon_tree.json'
+        assert content == truth_text
+        cached.append(True)
+
+    def mock_get(url, params):
+        assert url == 'https://www.genome.jp/kegg-bin/download_htext?htext=br08610'
+        assert params == {'format': 'json'}
+        return MockResponse(content=truth_text)
+
+    monkeypatch.setattr(io, 'load_cached_file', mock_get_cached_file)
+    monkeypatch.setattr(requests, 'get', mock_get)
+    monkeypatch.setattr(io, 'cache_file', mock_cache_file)
+
+    assert KEGGAnnotationIterator._get_taxon_tree() == truth
+    assert cached == [True]
+
+
+def test_kegg_annotation_iterator_get_taxon_tree_cached(monkeypatch):
+    truth = {"sample": "json", "lorem": "ipsum"}
+
+    def mock_get_cached_file(filename):
+        return '{"sample":"json","lorem":"ipsum"}'
+
+    monkeypatch.setattr(io, 'load_cached_file', mock_get_cached_file)
+    assert KEGGAnnotationIterator._get_taxon_tree() == truth
+
+
+def test_get_gui_videos(monkeypatch):
+    mock_dir = Path('tests/.pytest_cache/video_test_cache')
+    video_files = ('first_vid.webp', 'second_vid.webp', 'third_vid.webp')
+    visited = []
+
+    def mock_get(url, params):
+        base_url = 'https://github.com/GuyTeichman/RNAlysis/blob/master/rnalysis/gui/videos/'
+        assert url.startswith(base_url)
+        visited.append(url.split(base_url)[1])
+        return MockResponse(content='content')
+
+    monkeypatch.setattr(requests, 'get', mock_get)
+    monkeypatch.setattr(rnalysis.utils.io, 'get_tutorial_videos_dir',
+                        lambda: mock_dir)
+
+    try:
+        list(get_gui_videos(video_files))
+        assert sorted(visited) == ['first_vid.webp', 'second_vid.webp', 'third_vid.webp']
+        assert mock_dir.exists()
+        file_count = 0
+        for file in mock_dir.iterdir():
+            assert file.name in visited
+            with open(file) as f:
+                assert f.read() == 'content'
+            file_count += 1
+        assert file_count == len(video_files)
+    finally:
+        shutil.rmtree(mock_dir, ignore_errors=True)
